@@ -24,6 +24,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "util.h"
+
 /* Definitions to get compatibility with python 2.4 and earlier which
    does not have Py_ssize_t. See also PEP 353.
    Note: msvc (8 or earlier) does not have ssize_t, so we use Py_ssize_t.
@@ -35,13 +37,13 @@ typedef int Py_ssize_t;
 #endif
 
 #ifdef _WIN32
-# ifdef _MSC_VER
+#ifdef _MSC_VER
 /* msvc 6.0 has problems */
-#  define inline __inline
+#define inline __inline
 typedef unsigned long uint32_t;
-# else
-#  include <stdint.h>
-# endif
+#else
+#include <stdint.h>
+#endif
 static uint32_t ntohl(uint32_t x)
 {
 	return ((x & 0x000000ffUL) << 24) |
@@ -51,13 +53,13 @@ static uint32_t ntohl(uint32_t x)
 }
 #else
 /* not windows */
-# include <sys/types.h>
-# ifdef __BEOS__
-#  include <ByteOrder.h>
-# else
-#  include <arpa/inet.h>
-# endif
-# include <inttypes.h>
+#include <sys/types.h>
+#if defined __BEOS__ && !defined __HAIKU__
+#include <ByteOrder.h>
+#else
+#include <arpa/inet.h>
+#endif
+#include <inttypes.h>
 #endif
 
 static char mpatch_doc[] = "Efficient binary patching.";
@@ -236,7 +238,7 @@ static struct flist *decode(const char *bin, int len)
 	struct flist *l;
 	struct frag *lt;
 	const char *data = bin + 12, *end = bin + len;
-	char decode[12]; /* for dealing with alignment issues */
+	uint32_t decode[3]; /* for dealing with alignment issues */
 
 	/* assume worst case size, we won't have many of these lists */
 	l = lalloc(len / 12);
@@ -247,9 +249,9 @@ static struct flist *decode(const char *bin, int len)
 
 	while (data <= end) {
 		memcpy(decode, bin, 12);
-		lt->start = ntohl(*(uint32_t *)decode);
-		lt->end = ntohl(*(uint32_t *)(decode + 4));
-		lt->len = ntohl(*(uint32_t *)(decode + 8));
+		lt->start = ntohl(decode[0]);
+		lt->end = ntohl(decode[1]);
+		lt->len = ntohl(decode[2]);
 		if (lt->start > lt->end)
 			break; /* sanity check */
 		bin = data + lt->len;
@@ -373,12 +375,12 @@ patches(PyObject *self, PyObject *args)
 		result = NULL;
 		goto cleanup;
 	}
-	result = PyString_FromStringAndSize(NULL, outlen);
+	result = PyBytes_FromStringAndSize(NULL, outlen);
 	if (!result) {
 		result = NULL;
 		goto cleanup;
 	}
-	out = PyString_AsString(result);
+	out = PyBytes_AsString(result);
 	if (!apply(out, in, inlen, patch)) {
 		Py_DECREF(result);
 		result = NULL;
@@ -395,7 +397,7 @@ patchedsize(PyObject *self, PyObject *args)
 	long orig, start, end, len, outlen = 0, last = 0;
 	int patchlen;
 	char *bin, *binend, *data;
-	char decode[12]; /* for dealing with alignment issues */
+	uint32_t decode[3]; /* for dealing with alignment issues */
 
 	if (!PyArg_ParseTuple(args, "ls#", &orig, &bin, &patchlen))
 		return NULL;
@@ -405,9 +407,9 @@ patchedsize(PyObject *self, PyObject *args)
 
 	while (data <= binend) {
 		memcpy(decode, bin, 12);
-		start = ntohl(*(uint32_t *)decode);
-		end = ntohl(*(uint32_t *)(decode + 4));
-		len = ntohl(*(uint32_t *)(decode + 8));
+		start = ntohl(decode[0]);
+		end = ntohl(decode[1]);
+		len = ntohl(decode[2]);
 		if (start > end)
 			break; /* sanity check */
 		bin = data + len;
@@ -435,10 +437,34 @@ static PyMethodDef methods[] = {
 	{NULL, NULL}
 };
 
+#ifdef IS_PY3K
+static struct PyModuleDef mpatch_module = {
+	PyModuleDef_HEAD_INIT,
+	"mpatch",
+	mpatch_doc,
+	-1,
+	methods
+};
+
+PyMODINIT_FUNC PyInit_mpatch(void)
+{
+	PyObject *m;
+
+	m = PyModule_Create(&mpatch_module);
+	if (m == NULL)
+		return NULL;
+
+	mpatch_Error = PyErr_NewException("mpatch.mpatchError", NULL, NULL);
+	Py_INCREF(mpatch_Error);
+	PyModule_AddObject(m, "mpatchError", mpatch_Error);
+
+	return m;
+}
+#else
 PyMODINIT_FUNC
 initmpatch(void)
 {
 	Py_InitModule3("mpatch", methods, mpatch_doc);
 	mpatch_Error = PyErr_NewException("mpatch.mpatchError", NULL, NULL);
 }
-
+#endif
